""" 
Class for storing the letter dictionary. As well as applying formatting to the letters.
"""
from .letter_dict import letter_dict


class Letter:
    def __init__(self, spacing: int = 1):
        self.spacing = spacing
        self.letter_representation = letter_dict

    def set_spacing(self):
        """Set the spacing between letters in words"""
        self.letter_representation = {
            key: list(map(lambda row: row + [" "] * self.spacing, value))
            for key, value in self.letter_representation.items()
        }

    def get_letter_representation(self):
        return self.letter_representation



""" 
Functions that prepare a grid of letters to be printed on the terminal. New rows are 
added when the current row size is greater than the terminal width.
"""
import os
from typing import List


def store_sentence(words: List[str], letter_representation: dict[str, list[list[str]]]):
    terminal_width = os.get_terminal_size().columns
    sentence = []
    word_group = []
    current_width = 0
    for word in words:
        word_representation = []
        word_width = 0
        for letter in word:
            single_letter_representation = letter_representation[letter.upper()]
            word_representation.append(single_letter_representation)
            letter_width = (
                max(len("".join(line)) for line in single_letter_representation) + 1
            )
            word_width += letter_width
        if current_width + word_width + len(word_group) * 4 > terminal_width:
            sentence.append(word_group)
            word_group = [word_representation]
            current_width = word_width
        else:
            word_group.append(word_representation)
            current_width += word_width
    if word_group:
        sentence.append(word_group)
    return sentence


def print_sentence(word_group):
    for i in range(5):
        for word_representation in word_group:
            for letter_representation in word_representation:
                printed_string = "".join(letter_representation[i])
                print("\033[33m" + printed_string, end="")
            print("    ", end="")
        print()



from setuptools import setup, find_packages

setup(
    name="bigword",
    version="0.1.0",
    description="Print big words on a grid",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Casey Webb",
    url="https://github.com/caseyrwebb/bigword",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "bigword=bigword.__main__:bigword",
        ],
    },
    python_requires=">=3.10.5",
    install_requires=[],
)



""" 
Main file for the Bigword package.
"""
import argparse
import sys
import logging

from .grid import print_sentence, store_sentence
from .letter_class import Letter


def bigword():
    parser = argparse.ArgumentParser(description="Print words on a grid")

    parser.add_argument(
        "words",
        type=str,
        nargs="?",
        help="The sentence to print. Must pass in as a continuous string.\nExample: bigwords 'this is a bigg ol sentence'",
    )

    args = parser.parse_args()

    if args.words is None:
        logging.error("Please pass in a sentence to print")
        sys.exit(1)

    elif not isinstance(args.words, str):
        logging.error("Argument must be a string")
        sys.exit(1)

    l = Letter()
    l.set_spacing()
    letter_representation = l.get_letter_representation()

    words = args.words.split()

    sentence = store_sentence(words, letter_representation)
    for word_group in sentence:
        print_sentence(word_group)
        print()


if __name__ == "__main__":
    bigword()
